
class UndefinedConnectionError(Exception):
    """
    Raise exception when connection to service is not defined
    """
    pass


class ConvertValueError(Exception):
    """
    Raise exception when coversion of value is not possible
    """
    pass
